# my_auth_lib/auth.py
from datetime import datetime, timedelta, timezone
from jose import jwt
from fastapi import Depends, HTTPException
from fastapi.security import OAuth2PasswordBearer
from dotenv import load_dotenv
import os
from .hashing import verify_password
from database import get_connection   # Adjust the import as needed
from schemas import UserCreate        # Adjust the import as needed

# Load environment variables
load_dotenv()

# Secret key for JWT
SECRET_KEY = os.getenv("JWT_SECRET")
ALGORITHM = "HS256"

# FastAPI's OAuth2 scheme for token authentication
oauth2_scheme = OAuth2PasswordBearer(tokenUrl="login")

# Function to generate a JWT token
def create_access_token(data: dict, expires_delta: timedelta):
    to_encode = data.copy()
    expire = datetime.now(timezone.utc) + expires_delta
    to_encode.update({"exp": expire})
    return jwt.encode(to_encode, SECRET_KEY, algorithm=ALGORITHM)

# Function to handle user registration
def register_user(user: UserCreate):
    conn = get_connection()
    if conn is None:
        raise HTTPException(status_code=500, detail="Database Connection Error")
    
    cursor = conn.cursor()
    cursor.execute("SELECT * FROM users WHERE username=%s OR email=%s", (user.username, user.email))
    existing_user = cursor.fetchone()
    if existing_user:
        raise HTTPException(status_code=400, detail="Username or Email already exists")
    
    # Hash the password and insert new user
    from .hashing import hash_password
    hashed_password = hash_password(user.password)
    cursor.execute("INSERT INTO users (username, email, password) VALUES (%s, %s, %s)", 
                   (user.username, user.email, hashed_password))
    conn.commit()
    conn.close()
    return {"message": "User registered successfully"}

# Function to handle user login
def login_user(form_data):
    conn = get_connection()
    if conn is None:
        raise HTTPException(status_code=500, detail="Database connection error")
    
    cursor = conn.cursor()
    cursor.execute("SELECT id, username, email, password FROM users WHERE username = %s", (form_data.username,))
    user = cursor.fetchone()
    conn.close()

    if not user:
        raise HTTPException(status_code=400, detail="Invalid username or password")

    user_id, username, email, hashed_password = user

    if not verify_password(form_data.password, hashed_password):
        raise HTTPException(status_code=400, detail="Invalid username or password")

    access_token = create_access_token(
        data={"sub": username, "user_id": user_id},
        expires_delta=timedelta(hours=1)
    )

    return {"access_token": access_token, "token_type": "bearer"}
